from mongodb_controller import client
import pandas as pd

DATABASE_NAME_RPA = 'database-rpa'
COLLECTION_NAME_MENU2205 = 'dataset-menu2205'

collection_menu2205 = client[DATABASE_NAME_RPA][COLLECTION_NAME_MENU2205]

def create_pipeline_for_latest_date_of_menu2205(fund_code):
    pipeline = [
        {'$match': {'fund_code': fund_code}},
        {'$sort': {'date_ref': -1}},
        {'$project': {'_id': 0, 'date_ref': 1}},
        {'$limit': 1}
    ]
    return pipeline

def get_latest_date_ref_of_menu2205_by_fund_code(fund_code):
    pipeline = create_pipeline_for_latest_date_of_menu2205(fund_code)
    cursor = collection_menu2205.aggregate(pipeline)
    return list(cursor)[0]['date_ref']

def create_pipeline_for_menu2205(fund_code, date_ref=None):
    pipeline = [
        {'$match': {'fund_code': fund_code, 'date_ref': date_ref}},
        {'$project': {'_id': 0, 'data': 1}}
    ]
    return pipeline

def fetch_data_menu2205(fund_code, date_ref=None):
    date_ref = date_ref if date_ref else get_latest_date_ref_of_menu2205_by_fund_code(fund_code)
    pipeline = create_pipeline_for_menu2205(fund_code, date_ref)
    cursor = collection_menu2205.aggregate(pipeline)
    return list(cursor)[0]['data']

def fetch_df_menu2205(fund_code, date_ref=None):
    data = fetch_data_menu2205(fund_code, date_ref)
    df = pd.DataFrame(data)
    return df